@@ -336,6 +336,15 @@ if_end_update(void)
     }
 }
 
+void
+if_flush_ifaces(struct proto *p)
+{
+  if (p->debug & D_EVENTS)
+    log(L_TRACE "%s: Flushing interfaces", p->name);
+  if_start_update();
+  if_end_update();
+}
+
 /**
  * if_feed_baby - advertise interfaces to a new protocol
  * @p: protocol to feed
@@ -557,8 +566,8 @@ if_init(void)
  *	Interface Pattern Lists
  */
 
-static int
-iface_patt_match(struct iface_patt *ifp, struct iface *i)
+int
+iface_patt_match(struct iface_patt *ifp, struct iface *i, struct ifa *a)
 {
   struct iface_patt_node *p;
 
@@ -579,23 +588,32 @@ iface_patt_match(struct iface_patt *ifp, struct iface *i)
 	    continue;
 	}
 
-      if (p->pxlen)
-	if (!i->addr || !ipa_in_net(i->addr->ip, p->prefix, p->pxlen))
-	  continue;
+      if (p->pxlen == 0)
+	return pos;
 
-      return pos;
+      if (!a)
+	continue;
+
+      if (ipa_in_net(a->ip, p->prefix, p->pxlen))
+	return pos;
+
+      if ((a->flags & IA_UNNUMBERED) &&
+	  ipa_in_net(a->opposite, p->prefix, p->pxlen))
+	return pos;
+	  
+      continue;
     }
 
   return 0;
 }
 
 struct iface_patt *
-iface_patt_find(list *l, struct iface *i)
+iface_patt_find(list *l, struct iface *i, struct ifa *a)
 {
   struct iface_patt *p;
 
   WALK_LIST(p, *l)
-    if (iface_patt_match(p, i))
+    if (iface_patt_match(p, i, a))
       return p;
 
   return NULL;
