@@ -75,9 +75,13 @@ if_connected(ip_addr *a, struct iface *i) /* -1=error, 1=match, 0=no match */
 	{
 	  if (ipa_in_net(*a, b->prefix, b->pxlen))
 	    {
-	      if (ipa_equal(*a, b->prefix) ||	/* Network address */
-		  ipa_equal(*a, b->brd))	/* Broadcast */
+#ifndef IPV6
+	      if ((b->pxlen < (BITS_PER_IP_ADDRESS - 1)) &&
+		  (ipa_equal(*a, b->prefix) ||	/* Network address */
+		   ipa_equal(*a, b->brd)))	/* Broadcast */
 		return -1;
+#endif
+
 	      return b->scope;
 	    }
 	}
@@ -100,48 +104,71 @@ if_connected(ip_addr *a, struct iface *i) /* -1=error, 1=match, 0=no match */
  * IP address, neigh_find() returns %NULL.
  */
 
+
 neighbor *
 neigh_find(struct proto *p, ip_addr *a, unsigned flags)
+{
+  return neigh_find2(p, a, NULL, flags);
+}
+
+
+neighbor *
+neigh_find2(struct proto *p, ip_addr *a, struct iface *ifa, unsigned flags)
 {
   neighbor *n;
-  int class, scope = SCOPE_HOST;
+  int class, scope = -1;       ;
   unsigned int h = neigh_hash(p, a);
-  struct iface *i, *j;
+  struct iface *i;
 
   WALK_LIST(n, neigh_hash_table[h])	/* Search the cache */
-    if (n->proto == p && ipa_equal(*a, n->addr))
+    if (n->proto == p && ipa_equal(*a, n->addr) && (!ifa || (ifa == n->iface)))
       return n;
 
   class = ipa_classify(*a);
   if (class < 0)			/* Invalid address */
     return NULL;
-  if ((class & IADDR_SCOPE_MASK) < SCOPE_SITE ||
+  if (((class & IADDR_SCOPE_MASK) == SCOPE_HOST) ||
+      (((class & IADDR_SCOPE_MASK) == SCOPE_LINK) && (ifa == NULL)) ||
       !(class & IADDR_HOST))
     return NULL;			/* Bad scope or a somecast */
 
-  j = NULL;
-  WALK_LIST(i, iface_list)
-    if ((scope = if_connected(a, i)) >= 0)
-      {
-	j = i;
-	break;
-      }
-  if (!j && !(flags & NEF_STICKY))
+  if (ifa)
+    {
+      scope = if_connected(a, ifa);
+
+      if ((scope < 0) && (flags & NEF_ONLINK))
+	scope = class & IADDR_SCOPE_MASK;
+    }
+  else
+    WALK_LIST(i, iface_list)
+      if ((scope = if_connected(a, i)) >= 0)
+	{
+	  ifa = i;
+	  break;
+	}
+
+  /* scope < 0 means i don't know neighbor */
+  /* scope >= 0 implies ifa != NULL */
+
+  if ((scope < 0) && !(flags & NEF_STICKY))
     return NULL;
 
   n = sl_alloc(neigh_slab);
   n->addr = *a;
-  n->iface = j;
-  if (j)
+  if (scope >= 0)
     {
       add_tail(&neigh_hash_table[h], &n->n);
-      add_tail(&j->neighbors, &n->if_n);
+      add_tail(&ifa->neighbors, &n->if_n);
     }
   else
     {
+      /* sticky flag does not work for link-local neighbors;
+	 fortunately, we don't use this combination */
       add_tail(&sticky_neigh_list, &n->n);
+      ifa = NULL;
       scope = 0;
     }
+  n->iface = ifa;
   n->proto = p;
   n->data = NULL;
   n->aux = 0;
