@@ -37,6 +37,7 @@ struct fib_node {
   byte pxlen;
   byte flags;				/* User-defined */
   byte x0, x1;				/* User-defined */
+  u32 uid;				/* Unique ID based on hash */
   ip_addr prefix;			/* In host order */
 };
 
@@ -165,6 +166,7 @@ typedef struct rte {
     struct {
       u32 metric1, metric2;		/* OSPF Type 1 and Type 2 metrics */
       u32 tag;				/* External route tag */
+      u32 router_id;			/* Router that originated this route */
     } ospf;
 #endif
     struct {				/* Routes generated by krt sync (both temporary and inherited ones) */
