@@ -10,8 +10,12 @@
 #define CONFIG_SELF_CONSCIOUS
 #define CONFIG_MULTIPLE_TABLES
 #define CONFIG_ALL_TABLES_AT_ONCE
+#define CONFIG_MC_PROPER_SRC
+
+#undef CONFIG_SKIP_MC_BIND
 
 #define CONFIG_LINUX_MC_MREQN
+#define CONFIG_UNIX_DONTROUTE
 
 /*
 Link: sysdep/linux/netlink
