@@ -19,16 +19,18 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "nest/bird.h"
 #include "nest/cli.h"
+#include "nest/mrtdump.h"
 #include "lib/string.h"
 #include "lib/lists.h"
 #include "lib/unix.h"
 
-static FILE *dbgf = NULL;
+static FILE *dbgf;
 static list *current_log_list;
-static list init_log_list;
+static char *current_syslog_name; /* NULL -> syslog closed */
 
 bird_clock_t rate_limit_time = 5;
 int rate_limit_count = 5;
@@ -78,21 +80,13 @@ vlog(int class, char *msg, va_list args)
 	continue;
       if (l->fh)
 	{
-	  time_t now = time(NULL);
-	  struct tm *tm = localtime(&now);
-
 	  if (l->terminal_flag)
 	    fputs("bird: ", l->fh);
 	  else
 	    {
-	      fprintf(l->fh, "%02d-%02d-%04d %02d:%02d:%02d <%s> ",
-		       tm->tm_mday,
-		       tm->tm_mon+1,
-		       tm->tm_year+1900,
-		       tm->tm_hour,
-		       tm->tm_min,
-		       tm->tm_sec,
-		       class_names[class]);
+	      byte tbuf[TM_DATETIME_BUFFER_SIZE];
+	      tm_format_datetime(tbuf, &config->tf_log, now);
+	      fprintf(l->fh, "%s <%s> ", tbuf, class_names[class]);
 	    }
 	  fputs(buf, l->fh);
 	  fputc('\n', l->fh);
@@ -215,38 +209,55 @@ debug(char *msg, ...)
   va_end(args);
 }
 
-void
-log_init(int debug, int init)
+static list *
+default_log_list(int debug, int init, char **syslog_name)
 {
-  static struct log_config lc_stderr = { mask: ~0, terminal_flag: 1 };
-
+  static list init_log_list;
   init_list(&init_log_list);
-  current_log_list = &init_log_list;
+  *syslog_name = NULL;
 
 #ifdef HAVE_SYSLOG
   if (!debug)
     {
       static struct log_config lc_syslog = { mask: ~0 };
-      openlog("bird", LOG_CONS | LOG_NDELAY, LOG_DAEMON);
-      add_tail(current_log_list, &lc_syslog.n);
+      add_tail(&init_log_list, &lc_syslog.n);
+      *syslog_name = bird_name;
       if (!init)
-	return;
+	return &init_log_list;
     }
 #endif
 
+  static struct log_config lc_stderr = { mask: ~0, terminal_flag: 1 };
   lc_stderr.fh = stderr;
-  add_tail(current_log_list, &lc_stderr.n);
+  add_tail(&init_log_list, &lc_stderr.n);
+  return &init_log_list;
 }
 
 void
-log_switch(int debug, list *l)
+log_switch(int debug, list *l, char *new_syslog_name)
 {
-  if (EMPTY_LIST(*l))
-    log_init(debug, 0);
-  else
-    current_log_list = l;
+  if (!l || EMPTY_LIST(*l))
+    l = default_log_list(debug, !l, &new_syslog_name);
+
+  current_log_list = l;
+
+#ifdef HAVE_SYSLOG
+  if (current_syslog_name && new_syslog_name &&
+      !strcmp(current_syslog_name, new_syslog_name))
+    return;
+
+  if (current_syslog_name)
+    closelog();
+
+  if (new_syslog_name)
+    openlog(new_syslog_name, LOG_CONS | LOG_NDELAY, LOG_DAEMON);
+
+  current_syslog_name = new_syslog_name;
+#endif
 }
 
+
+
 void
 log_init_debug(char *f)
 {
@@ -261,3 +272,16 @@ log_init_debug(char *f)
   if (dbgf)
     setvbuf(dbgf, NULL, _IONBF, 0);
 }
+
+void
+mrt_dump_message(struct proto *p, u16 type, u16 subtype, byte *buf, u32 len)
+{
+  /* Prepare header */
+  put_u32(buf+0, now_real);
+  put_u16(buf+4, type);
+  put_u16(buf+6, subtype);
+  put_u32(buf+8, len - MRTDUMP_HDR_LENGTH);
+
+  if (p->cf->global->mrtdump_file != -1)
+    write(p->cf->global->mrtdump_file, buf, len);
+}
