@@ -83,7 +83,7 @@ sysdep_preconfig(struct config *c)
 int
 sysdep_commit(struct config *new, struct config *old UNUSED)
 {
-  log_switch(debug_flag, &new->logfiles);
+  log_switch(debug_flag, &new->logfiles, new->syslog_name);
   return 0;
 }
 
@@ -141,6 +141,9 @@ cmd_reconfig(char *name, int type)
 {
   struct config *conf;
 
+  if (cli_access_restricted())
+    return;
+
   if (!name)
     name = config_name;
   cli_msg(-2, "Reading configuration from %s", name);
@@ -295,14 +298,23 @@ cli_init_unix(void)
   s->type = SK_UNIX_PASSIVE;
   s->rx_hook = cli_connect;
   s->rbsize = 1024;
-  if (sk_open_unix(s, path_control_socket) < 0)
-    die("Unable to create control socket %s", path_control_socket);
+  sk_open_unix(s, path_control_socket);
 }
 
 /*
  *	Shutdown
  */
 
+void
+cmd_shutdown(void)
+{
+  if (cli_access_restricted())
+    return;
+
+  cli_msg(7, "Shutdown requested");
+  order_shutdown();
+}
+
 void
 async_shutdown(void)
 {
@@ -313,8 +325,9 @@ async_shutdown(void)
 void
 sysdep_shutdown_done(void)
 {
-  unlink(PATH_CONTROL_SOCKET);
-  die("System shutdown completed");
+  unlink(path_control_socket);
+  log_msg(L_FATAL "Shutdown completed");
+  exit(0);
 }
 
 /*
@@ -364,20 +377,37 @@ signal_init(void)
  *	Parsing of command-line arguments
  */
 
-static char *opt_list = "c:dD:s:";
+static char *opt_list = "c:dD:ps:";
+static int parse_and_exit;
+char *bird_name;
 
 static void
 usage(void)
 {
-  fprintf(stderr, "Usage: bird [-c <config-file>] [-d] [-D <debug-file>] [-s <control-socket>]\n");
+  fprintf(stderr, "Usage: %s [-c <config-file>] [-d] [-D <debug-file>] [-p] [-s <control-socket>]\n", bird_name);
   exit(1);
 }
 
+static inline char *
+get_bird_name(char *s, char *def)
+{
+  char *t;
+  if (!s)
+    return def;
+  t = strrchr(s, '/');
+  if (!t)
+    return s;
+  if (!t[1])
+    return def;
+  return t+1;
+}
+
 static void
 parse_args(int argc, char **argv)
 {
   int c;
 
+  bird_name = get_bird_name(argv[0], "bird");
   if (argc == 2)
     {
       if (!strcmp(argv[1], "--version"))
@@ -401,6 +431,9 @@ parse_args(int argc, char **argv)
 	log_init_debug(optarg);
 	debug_flag |= 2;
 	break;
+      case 'p':
+	parse_and_exit = 1;
+	break;
       case 's':
 	path_control_socket = optarg;
 	break;
@@ -426,9 +459,10 @@ main(int argc, char **argv)
   parse_args(argc, argv);
   if (debug_flag == 1)
     log_init_debug("");
-  log_init(debug_flag, 1);
+  log_switch(debug_flag, NULL, NULL);
 
-  test_old_bird(path_control_socket);
+  if (!parse_and_exit)
+    test_old_bird(path_control_socket);
 
   DBG("Initializing.\n");
   resource_init();
@@ -437,12 +471,18 @@ main(int argc, char **argv)
   rt_init();
   if_init();
 
+  if (!parse_and_exit)
+    cli_init_unix();
+
   protos_build();
   proto_build(&proto_unix_kernel);
   proto_build(&proto_unix_iface);
 
   read_config();
 
+  if (parse_and_exit)
+    exit(0);
+
   if (!debug_flag)
     {
       pid_t pid = fork();
@@ -460,12 +500,11 @@ main(int argc, char **argv)
 
   signal_init();
 
-  cli_init_unix();
-
 #ifdef LOCAL_DEBUG
   async_dump_flag = 1;
 #endif
 
+  log(L_INFO "Started");
   DBG("Entering I/O loop.\n");
 
   io_loop();
