@@ -33,14 +33,17 @@ void tm_dump_all(void);
 extern bird_clock_t now; 		/* Relative, monotonic time in seconds */
 extern bird_clock_t now_real;		/* Time in seconds since fixed known epoch */
 
+struct timeformat {
+  char *fmt1, *fmt2;
+  bird_clock_t limit;
+};
+
 bird_clock_t tm_parse_date(char *);	/* Convert date to bird_clock_t */
 bird_clock_t tm_parse_datetime(char *);	/* Convert date to bird_clock_t */
-void tm_format_date(char *, bird_clock_t);	/* Convert bird_clock_t to date */
-#define TM_DATE_BUFFER_SIZE 12		/* Buffer size required by tm_format_date */
-void tm_format_datetime(char *, bird_clock_t);	/* Convert bird_clock_t to date + time */
-#define TM_DATETIME_BUFFER_SIZE 64	/* Buffer size required by tm_format_datetime */
-void tm_format_reltime(char *, bird_clock_t);	/* Convert bird_clock_t to relative datetime string */
-#define TM_RELTIME_BUFFER_SIZE 12	/* Buffer size required by tm_format_reltime */
+
+#define TM_DATETIME_BUFFER_SIZE 32	/* Buffer size required by tm_format_datetime */
+void
+tm_format_datetime(char *x, struct timeformat *fmt_spec, bird_clock_t t);
 
 #ifdef TIME_T_IS_64BIT
 #define TIME_INFINITY 0x7fffffffffffffff
